package com.bwie.gateway.filters;

import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import io.jsonwebtoken.Claims;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpMethod;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.concurrent.TimeUnit;

@Component
public class AuthFilter implements GlobalFilter, Ordered {

    private final IgnoreWhiteConfig ignoreWhiteConfig;
    private final RedisCache redisCache;

    public AuthFilter(IgnoreWhiteConfig ignoreWhiteConfig, RedisCache redisCache) {
        this.ignoreWhiteConfig = ignoreWhiteConfig;
        this.redisCache = redisCache;
    }

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        ServerHttpRequest.Builder mutate = request.mutate();
        String path = request.getURI().getPath();
        System.out.println("path = " + path);
        HttpMethod method = request.getMethod();
        System.out.println("method = " + method);
        if(StringUtils.matches(path,ignoreWhiteConfig.getWhites())){
            return chain.filter(exchange);
        }
        String first = request.getHeaders().getFirst(TokenConstants.TOKEN);

        if(null==first){
            return GatewayUtils.errorResponse(exchange,"token不为空");
        }

        Claims claims = JwtUtils.parseToken(first);
        if(null==claims){
            return GatewayUtils.errorResponse(exchange,"token格式不正确");
        }
        String userKey = JwtUtils.getUserKey(claims);
        if(!redisCache.hasKey(TokenConstants.LOGIN_TOKEN_KEY+userKey)){
            return GatewayUtils.errorResponse(exchange,"token过期");
        }
        redisCache.expire(TokenConstants.LOGIN_TOKEN_KEY+userKey,30, TimeUnit.MINUTES);

        String userId = JwtUtils.getUserId(claims);
        String userName = JwtUtils.getUserName(claims);
        GatewayUtils.addHander(mutate, JwtConstants.USER_KEY,userKey);
        GatewayUtils.addHander(mutate,JwtConstants.DETAILS_USERNAME,userName);
        GatewayUtils.addHander(mutate,JwtConstants.DETAILS_USER_ID,userId);
        GatewayUtils.removeHeader(mutate,TokenConstants.TOKEN);

        return chain.filter(exchange.mutate().request(mutate.build()).build());
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
